<html>
  <head>
    <title>Links for {{ package.name }}</title>
  </head>
  <body>

    <h1>Links for {{ package.name }}</h1>
    {% for release in package.releases %}
      {% for file in release.files %}
      <a href="{{ request.route_url('download_package', package=package.name, basename=file.filename, kind=file.kind, letter=letter) }}?md5={{ file.md5 }}">{{ file.filename }}</a>
        <br>
      {% endfor %}
    {% endfor %}
  </body>
</html>

