<html>
  <head>
    <title>Simple index</title>
  </head>
  <body>

    {% for package in packages %}
    <a href="{{ request.route_url('package_list', package=package.name) }}">{{ package.name }}</a>
    <br>
    {% endfor %}
  </body>
</html>
